package com.hibernate.util;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.Session;
import org.hibernate.cfg.Configuration;
import org.hibernate.jdbc.ReturningWork;

import com.hibernate.model.Employee;

public class AppMain {

	public static void main(String[] args) {		
		// Creating the configuration instance & passing the hibernate configuration file.
		Configuration config = new Configuration();
		config.configure("hibernate.cfg.xml");

		// Hibernate session object to start the db transaction.
		Session s = config.buildSessionFactory().openSession();

		// Fetching the data from the database using the Hibernate's ReturningWork interface.
		// Hibernate's doReturningWork() method performs the SELECT operations in the database!
		List<Employee> emp = null;

		// Just pass object to ReturningWork<T> interface.
		emp = s.doReturningWork(new ReturningWork<List<Employee>>() {
			@Override
			public List<Employee> execute(Connection connection) throws SQLException {

				Employee e = null;
				CallableStatement cstmt= null;
				List<Employee> inrList = new ArrayList<Employee>();
				try {
					String sqlString= "{call returnEmployeeList}";
					cstmt = connection.prepareCall(sqlString);

					ResultSet rs = cstmt.executeQuery();
					while(rs.next()) {
						e = new Employee(rs.getInt("eid"), rs.getString("ename"), rs.getString("ecompany"), rs.getString("edept"), rs.getString("egender"));
						inrList.add(e);
					}
				} finally {
					cstmt.close();
				}
				return inrList;
			}
		});

		// Displaying the employee records returned from the database table.
		if(emp != null && emp.size() > 0) {
			for(Employee e : emp) {
				System.out.println(e.toString());
			}
		} else {
			System.out.println("No employee record exists in the table.");
		}

		// Closing the session object.
		s.close();
	}
}